@@ -1,6 +1,7 @@
 import json
 import os
 import subprocess
+from typing import Callable, List
 
 from bench_wizard.config import Config
 
@@ -42,7 +43,19 @@ class Benchmark:
         self._acceptable = False
         self._rerun = False
 
-    def run(self, rerun=False):
+    @property
+    def acceptable(self) -> bool:
+        return self._acceptable
+
+    @acceptable.setter
+    def acceptable(self, value: bool):
+        self._acceptable = value
+
+    @property
+    def raw(self) -> bytes:
+        return self._stdout
+
+    def run(self, rerun: bool = False) -> None:
         result = subprocess.run(self._command, capture_output=True)
         # TODO: check the return code
 
@@ -57,7 +70,7 @@ class Benchmark:
                 extrinsic = info[1].split(":")[1].strip()[1:-1]
                 if extrinsic in self._extrinsics:
                     self._extrinsics_results.append(
-                        process_extrinsic(lines[idx + 1 : idx + 21])
+                        process_extrinsic(lines[idx + 1: idx + 21])
                     )
 
         self._total_time = sum(list(map(lambda x: float(x), self._extrinsics_results)))
@@ -65,31 +78,53 @@ class Benchmark:
 
         diff = int(self._ref_value - self._total_time)
 
-        self._acceptable = diff >= -margin
+        self.acceptable = diff >= -margin
         self._rerun = rerun
 
-    def dump(self, dest):
+    def dump(self, dest: str) -> None:
         with open(os.path.join(dest, f"{self._pallet}.results"), "wb") as f:
             f.write(self._stdout)
 
+    def result_as_str(self) -> str:
+        # TODO: refactor to be nice and simple
+        pallet = self._pallet
+        ref_value = self._ref_value
+        current = self._total_time
 
-def load_ref_values(filename):
-    with open(filename, "r") as f:
-        return json.load(f)
+        margin = int(ref_value * DIFF_MARGIN / 100)
+
+        diff = int(ref_value - current)
+
+        percentage = f"{(diff / (ref_value + current)) * 100:.2f}"
+
+        note = "OK" if diff >= -margin else "FAILED"
+
+        diff = f"{diff}"
+        times = f"{ref_value:.2f} vs {current:.2f}"
+
+        rerun = "*" if self._rerun else ""
+
+        return f"{pallet:<25}| {times:^25} | {diff:^14}| {percentage:^14} | {note:^10} | {rerun:^10}"
 
 
-def process_extrinsic(data):
+def process_extrinsic(data: List[str]) -> float:
     for entry in data:
         if entry.startswith("Time"):
             return float(entry.split(" ")[-1])
 
 
-def prepare_benchmarks(config: Config, reference_values: dict):
-
+def _prepare_benchmarks(config: Config, reference_values: dict) -> List[Benchmark]:
     benchmarks = []
 
     for pallet in config.pallets:
         command = COMMAND + [f"--pallet={pallet}"]
+        if config.output_dir:
+            output_file = os.path.join(config.output_dir, f"{pallet}.rs")
+            command += [f"--output={output_file}"]
+
+        if config.template:
+            command += [f"--template={config.template}"]
+
         ref_data = reference_values[pallet]
         ref_value = sum(list(map(lambda x: float(x), ref_data.values())))
         benchmarks.append(Benchmark(pallet, command, ref_value, ref_data.keys()))
@@ -97,73 +132,56 @@ def prepare_benchmarks(config: Config, reference_values: dict):
     return benchmarks
 
 
-def run_benchmarks(benchmarks: [Benchmark], rerun=False):
+def _run_benchmarks(benchmarks: List[Benchmark], rerun=False) -> None:
     # Note : this can be simplified into one statement
     if rerun:
-        [bench.run(rerun) for bench in benchmarks if bench._acceptable is False]
+        [bench.run(rerun) for bench in benchmarks if bench.acceptable is False]
     else:
-        print("Running benchmarks - this may take a while...")
         [bench.run() for bench in benchmarks]
 
 
-def show_pallet_result(pallet_result: Benchmark):
-    pallet = pallet_result._pallet
-    ref_value = pallet_result._ref_value
-    current = pallet_result._total_time
-
-    margin = int(ref_value * DIFF_MARGIN / 100)
-
-    diff = int(ref_value - current)
-
-    percentage = f"{(diff / (ref_value + current) ) * 100:.2f}"
-
-    note = "OK" if diff >= -margin else "FAILED"
-
-    diff = f"{diff}"
-    times = f"{ref_value:.2f} vs {current:.2f}"
-
-    rerun = "*" if pallet_result._rerun else ""
-
-    print(
-        f"{pallet:<25}| {times:^25} | {diff:^14}| {percentage:^14} | {note:^10} | {rerun:^10}"
-    )
-
-
-def run_pallet_benchmarks(config: Config):
+def run_pallet_benchmarks(config: Config, to_output: Callable[[str], None]) -> None:
     if not config.do_pallet_bench:
         return
 
-    print("Substrate Node Performance check ... ")
+    to_output("Substrate Node Performance check ... ")
 
     if config.do_pallet_bench:
-        s = load_ref_values(config.reference_values)
+        with open(config.reference_values, "r") as f:
+            s = json.load(f)
 
-        benchmarks = prepare_benchmarks(config, s)
-        run_benchmarks(benchmarks)
+        benchmarks = _prepare_benchmarks(config, s)
 
-        if [b._acceptable for b in benchmarks].count(False) == 1:
-            # of ony failed - rerun it
-            run_benchmarks(benchmarks, True)
+        to_output("Running benchmarks - this may take a while...")
+        _run_benchmarks(benchmarks)
 
-        print("\nResults:\n\n")
+        if [b.acceptable for b in benchmarks].count(False) == 1:
+            # if only one failed - rerun it
+            _run_benchmarks(benchmarks, True)
 
-        print(
+        to_output("\nResults:\n\n")
+
+        to_output(
             f"{'Pallet':^25}|{'Time comparison (µs)':^27}|{'diff* (µs)':^15}|{'diff* (%)':^16}|{'': ^12}| {'Rerun': ^10}"
         )
 
         for bench in benchmarks:
-            show_pallet_result(bench)
+            to_output(bench.result_as_str())
+
+            if not config.performance_check:
+                # TODO: consolidate the check mess here ( there are too many flags )
+                print(bench.raw.decode("utf-8"))
 
             if config.dump_results:
                 bench.dump(config.dump_results)
 
-        print("\nNotes:")
-        print(
+        to_output("\nNotes:")
+        to_output(
             "* - diff means the difference between reference total time and total benchmark time of current machine"
         )
-        print(
+        to_output(
             f"* - if diff > {DIFF_MARGIN}% of ref value -> performance is same or better"
         )
-        print(
+        to_output(
             f"* - If diff < {DIFF_MARGIN}% of ref value -> performance is worse and might not be suitable to run node ( You may ask node devs for further clarifications)"
         )
